/*  
  The circuit:
  
 3.3 V LCD
 * (1) Vss pin to ground
 * (2) Vdd pin to 3.3V
 * (3) VO pin to ground via 10k resistor     
 * (4) RS pin to digital pin 9
 * (5) R/W pin to ground                      
 * (6) E pin to digital pin 8
 * (7) DB0 N/C
 * (8) DB1 N/C
 * (9) DB2 N/C
 * (10) DB3 N/C
 * (11) DB4 pin to digital pin 4
 * (12) DB5 pin to digital pin 5
 * (13) DB6 pin to digital pin 6
 * (14) DB7 pin to digital pin 7
 * (15) LED_K pin to 3.3V
 * (16) LED_A pin to ground
  
  
  Si4703 breakout board
 * +3.3V to 3.3V   
 * GND to ground 
 * SDIO pin to analog pin 4       [Yellow wire]
 * SCLK pin to analog pin 5       [Blue wire]
 * SEN pin N/C
 * RST pin to digital pin 2       [White wire]
 * GPIO1 pin N/C                
 * GPIO2 pin N/C                 
 
  Momentary (N/O) Pushbutton Inputs
 * Momentary pushbutton 1 +3.3V____button____digital pin 10 (seek down) with 10kohm pull down resistor to ground
 * Momentary pushbutton 2 +3.3V____button____digital pin 11 (seek up) with 10kohm pull down resistor to ground
 */
 
// include the library code:
#include <LiquidCrystal.h>
#include <SparkFunSi4703.h>
#include <Wire.h>

// define the pins for the LCD 
int DB4 = 4;
int DB5 = 5;
int DB6 = 6;
int DB7 = 7;
int E = 8;
int RS = 9;

// define the pins for the Si4703
int resetPin = 2;
int SDIO = A4;
int SCLK = A5;

// define the pins for the seek push buttons
int channelDown = 10;
int channelUp = 11;

// initialize the library with the numbers of the interface pins
// LCD initialisation
LiquidCrystal lcd(RS, E, DB4, DB5, DB6, DB7);
// Si4703 initialisation
Si4703_Breakout radio(resetPin, SDIO, SCLK);

// Define the variables for the radio
int channel;
int volume;

void setup() { 
  // define pushbutton pins as inputs
  pinMode(channelDown,INPUT);
  pinMode(channelUp,INPUT);
  delay(1000); //delay to account for powering up of Arduino and lag to get LCD running
  // set up the LCD's number of columns and rows: 
  lcd.begin(16, 2);
  // turn on the radio and set the volume to a desired level
  radio.powerOn();
  radio.setVolume(15); //volume between 0 and 15 
  // set the radio to a default frequency (Nova 96.9 MHz FM)
  channel = 969;
  radio.setChannel(channel);
  // display a title 'FM Radio' on line 0 of the LCD, centred
  // (note: line 1 is the second row, since counting begins with 0):  
  lcd.setCursor(0,0);
  lcd.print("    FM Radio    ");
  // show the channel frequency on the LCD, centred
  lcd.setCursor(0,1);
  lcd.print("    96.9 MHz    ");
}

void loop() {
  // check if the channelUp button has been pressed, if so seek up for a channel
  if (digitalRead(channelUp) == HIGH)
  {
    channel = radio.seekUp();
    updateLCD();
  }
  // check if the channelDown button has been pressed, if so seek down for a channel
  else if (digitalRead(channelDown) == HIGH)
  {
    channel = radio.seekDown();
    updateLCD();
  }
}

void updateLCD()
{
  //convert the integer channel into a string to add decimal place and "MHz" before displaying on LCD
  String channelString = String(channel); 
  String frequency;
    if (channelString.length() == 3) //if the frequency is 3 digits, place a decimal point after the second digit
    {
        frequency = channelString.substring(0,2) + "." + channelString.substring(2) + " MHz";
    }
    else if (channelString.length() == 4) //if the frequency is 4 digits, place a decimal point after the third digit
    {
        frequency = channelString.substring(0,3) + "." + channelString.substring(3) + " MHz";
    }
    
    //first clear line 1 of the display to account for transition between 3 and 4 digit frequencies
    lcd.setCursor(0,1);
    lcd.print("                ");
    //update the frequency on the LCD
    lcd.setCursor(4,1);
    lcd.print(frequency);
}
